#!/usr/bin/env python3
from kubernetes import client, config
from kubernetes.client.rest import ApiException
import logging
import os
import conf

logger = logging.getLogger(__name__)


def get_opa_config(
    namespace="osdu-services", name="opa-config", config_only=True, pretty=True
):
    # Configs can be set in Configuration class directly or using helper utility

    try:
        if (
            conf.CLOUD_PROVIDER == conf.LOCAL
            or conf.CLOUD_PROVIDER == conf.MOCK
            or "KUBERNETES_SERVICE_HOST" not in os.environ
        ):
            logger.info(
                f"kubernetes load kube config {conf.CLOUD_PROVIDER} {namespace} {name}"
            )
            config.load_kube_config()
        else:
            """
            Use the service account kubernetes gives to pods to connect to kubernetes cluster.
            It's intended for clients that expect to be running inside a pod running on kubernetes.
            It will raise an exception if called from a process not running in a kubernetes environment.
            """
            logger.info(
                f"kubernetes load incluster config {conf.CLOUD_PROVIDER} {namespace} {name}"
            )
            config.load_incluster_config()
    except config.config_exception.ConfigException as err:
        logger.error(
            f"Exception when calling kubernetes load config {conf.CLOUD_PROVIDER}: {err}"
        )
        return None
    except Exception as e:
        logger.error(
            f"Unknown exception {e} when calling kubernetes load config {conf.CLOUD_PROVIDER}"
        )
        return None

    v1 = client.CoreV1Api()

    try:
        api_response = v1.read_namespaced_config_map(
            namespace=namespace, name=name, pretty=pretty
        )
        if config_only:
            if "opa-config.yaml" in api_response.data:
                return api_response.data["opa-config.yaml"]
            elif "config.yaml" in api_response.data:
                return api_response.data["config.yaml"]
            elif "config" in api_response.data:
                return api_response.data["config"]
            else:
                logger.error(f"Exception {api_response.data}")
                return None
        else:
            return api_response
    except ApiException as err:
        # If you get 403 errors from the API server you will have to configure K8s RBAC to add permission
        logger.error(
            f"Exception when calling CoreV1Api->list_namespaced_config_map: {err}"
        )
    except Exception as e:
        logger.error(
            f"Unknown exception {e} when calling CoreV1Api->list_namespaced_config_map"
        )
    return None


def patch_opa_config(
    namespace="osdu-services", name="opa-config", pretty=True, body=None
):
    try:
        if (
            conf.CLOUD_PROVIDER == conf.LOCAL
            or conf.CLOUD_PROVIDER == conf.MOCK
            or "KUBERNETES_SERVICE_HOST" not in os.environ
        ):
            logger.info(
                f"kubernetes load kube config {conf.CLOUD_PROVIDER} {namespace} {name}"
            )
            config.load_kube_config()
        else:
            """
            Use the service account kubernetes gives to pods to connect to kubernetes cluster.
            It's intended for clients that expect to be running inside a pod running on kubernetes.
            It will raise an exception if called from a process not running in a kubernetes environment.
            """
            logger.info(
                f"kubernetes load incluster config {conf.CLOUD_PROVIDER} {namespace} {name}"
            )
            config.load_incluster_config()
    except config.config_exception.ConfigException as err:
        logger.error(
            f"Exception when calling kubernetes load config {conf.CLOUD_PROVIDER}: {err}"
        )

    v1 = client.CoreV1Api()
    logger.info(f"loaded k8s config {namespace} {name} {str(body)}")

    try:
        api_response = v1.patch_namespaced_config_map(
            namespace=namespace, name=name, body=body, pretty=pretty
        )
        return api_response, None
    except ApiException as err:
        # If you get 403 errors from the API server you will have to configure K8s RBAC to add permission
        logger.error(
            f"Exception when calling CoreV1Api->update_namespaced_config_map: {err}"
        )
        return None, err
    return None, None


if __name__ == "__main__":
    data_partition = "osdu"
    name = "opa-config"
    r = get_opa_config(name=name, namespace=conf.NAMESPACE)
    if (
        r
        and f"osdu/partition/{data_partition}" in r
        and f"bundle-{data_partition}.tar.gz" in r
    ):
        print(f"found '{data_partition}' in configmap {name} in {conf.NAMESPACE}")
    else:
        print(f"not found '{data_partition}' in configmap {name} in {conf.NAMESPACE}")
