import pytest
import unittest
from fastapi.testclient import TestClient
import logging
import sys
import os
import json
from parameterized import parameterized

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

client = TestClient(app)


class TryUnitTesting(unittest.TestCase):
    def test_read_main_home_page(self):
        """Check Home Page"""
        response = client.get("/")
        self.assertEqual(response.status_code, 200, "Home Page is available")
        self.assertIn(
            "Policy Service", response.text, "Expected policy service text on Home Page"
        )

    def test_health_api_endpoint(self):
        """Check Health API"""
        response = client.get(conf.SERVICE_BASE_PATH + "/health")
        # Allow 503 for when OPA isn't available in unit testing
        assert response.status_code == 200 or response.status_code == 503
        self.assertIn(
            "Healthy", response.text, "Expected Healthy in response text from /health"
        )

    def test_docs_api_endpoint(self):
        """Check Docs endpoint"""
        response = client.get(conf.SERVICE_BASE_PATH + "/docs")
        self.assertEqual(response.status_code, 200, "Expected /docs to be available")

    def test_redocs_api_endpoint(self):
        """Check ReDocs endpoint"""
        response = client.get(conf.SERVICE_BASE_PATH + "/redoc")
        self.assertEqual(response.status_code, 200, "Expected Redoc to be available")

    def test_openapi_json_endpoint(self):
        """Check openapi json is available"""
        response = client.get(conf.SERVICE_BASE_PATH + "/openapi.json")
        self.assertEqual(
            response.status_code, 200, "Expected OpenAPI.json to be available"
        )

    def requires_auth(self, method, url, api_path=None):
        """Should always return 403"""
        with self.subTest(URL=url):
            response = method(url)
            if response.status_code == 405:
                pytest.xfail("method not allowed")
            self.assertEqual(
                response.status_code,
                403,
                f"Not sending token to our API should result in 403 Forbidden {url} but got {response.status_code}",
            )

    @parameterized.expand(
        [
            ["/policies", "get"],
            ["/policies/xyz", "get"],
            ["/policies/osdu/instance/xyz", "get"],
            ["/policies/osdu/partition/xyz/xyz", "get"],
            ["/policies/osdu/partition/xyz/xyz", "put"],
            ["/policies/osdu/partition/xyz/xyz", "delete"],
            ["/evaluations/query", "post"],
            ["/translate", "post"],
            ["/validate/xyz.rego", "put"],
            ["/compile", "post"],
            ["/config", "get"],
            ["/backup", "get"],
            ["/tenant", "get"],
            ["/tenant", "put"],
            ["/tenant", "delete"],
            ["/bootstrap", "post"],
        ]
    )
    def test_api_endpoints_make_sure_they_require_auth(self, api_path, method):
        if "get" in method:
            self.requires_auth(client.get, conf.SERVICE_BASE_PATH + api_path)
        elif "put" in method:
            self.requires_auth(client.put, conf.SERVICE_BASE_PATH + api_path)
        elif "delete" in method:
            self.requires_auth(client.delete, conf.SERVICE_BASE_PATH + api_path)
        elif "post" in method:
            self.requires_auth(client.post, conf.SERVICE_BASE_PATH + api_path)
        else:
            pytest.skip(f"Misconfigured method {method}")

    # @unittest.skipIf(conf.ENABLE_DEV_DIAGNOSTICS == False, "Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")
    @pytest.mark.opa_access
    @set_authorize_session(ADMIN_ONLY_SVC)
    def test_local_config_as_admin_only(self):
        """Check config API as a member of only admin group"""
        r = client.get(conf.SERVICE_BASE_PATH + "/config")
        print(f"config text: {r.text}")
        print(f"text: {r.text}", file=sys.stderr)
        self.assertEqual(r.status_code, 200, "Config Page is available")

    @pytest.mark.opa_access
    # @unittest.skipIf(conf.ENABLE_DEV_DIAGNOSTICS == False, "Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")
    @set_authorize_session(USER_AND_ADMIN_SVC)
    def test_config_as_user_and_admin(self):
        """Check config API as a member of both user and admin groups"""
        r = client.get(conf.SERVICE_BASE_PATH + "/config")
        self.assertEqual(r.status_code, 200, "Expected config Page is available")
        js = json.dumps(r.json(), indent=4)
        print(f"{js}")

    # @unittest.skipIf(conf.ENABLE_DEV_DIAGNOSTICS == False, "Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")
    @set_authorize_session(USER_ONLY_SVC)
    def test_config_as_user_only(self):
        """Check config API as member of user group"""
        r = client.get(conf.SERVICE_BASE_PATH + "/config")
        print(f"text: {r.text}")
        self.assertEqual(
            r.status_code, 403, "Expected config Page is available but forbidden"
        )

    @pytest.mark.opa_access
    # @unittest.skipIf(conf.ENABLE_DEV_DIAGNOSTICS == False, "Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")
    @set_authorize_session(OTHER_ONLY_SVC)
    def test_config_as_other(self):
        """Check config API as not a member of user and admin groups"""
        r = client.get(conf.SERVICE_BASE_PATH + "/config")
        self.assertEqual(
            r.status_code, 403, "Expected config is available but forbidden"
        )
        print(f"text: {r.text}")

    # @unittest.skipIf(conf.ENABLE_DEV_DIAGNOSTICS == False, "Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")
    @set_authorize_session()
    def test_config_as_none(self):
        """Check config API as not a member of any group"""
        r = client.get(conf.SERVICE_BASE_PATH + "/config")
        print(f"text: {r.text}")
        self.assertEqual(
            r.status_code, 403, "Expected config is available but forbidden"
        )

    @set_authorize_session(USER_ONLY_SVC)
    def test_info(self):
        """Check info API endpoint"""
        r = client.get(conf.SERVICE_BASE_PATH + "/info")
        print(f"text: {r.text}")
        self.assertEqual(r.status_code, 200, "Expected /info is available")

    @set_authorize_session(USER_ONLY_SVC)
    def test_mock_read_policies(self):
        """Check /policies API endpoint"""
        real_cloud_provider = os.getenv("CLOUD_PROVIDER", conf.MOCK)
        os.environ["CLOUD_PROVIDER"] = conf.MOCK
        r = client.get(conf.SERVICE_BASE_PATH + "/policies")
        print(f"text: {r.text}")
        self.assertEqual(r.status_code, 200, "Expected /policies is available")
        os.environ["CLOUD_PROVIDER"] = real_cloud_provider

    # at some point this should be expanded to test permissions
    @set_authorize_session(OTHER_ONLY_SVC, "other_partition_id")
    def test_mock_read_a_policy(self):
        """Check /policies API endpoint"""
        real_cloud_provider = os.getenv("CLOUD_PROVIDER", conf.MOCK)
        os.environ["CLOUD_PROVIDER"] = conf.MOCK
        r = client.get(conf.SERVICE_BASE_PATH + "/policies/xyz")
        print(f"text: {r.text}")
        self.assertEqual(r.status_code, 404, "Not expected to be found")
        os.environ["CLOUD_PROVIDER"] = real_cloud_provider

    @set_authorize_session(ADMIN_ONLY_SVC, "osdu")
    def test_mock_delete_a_policy(self):
        """Check delete /policies API endpoint"""
        real_cloud_provider = os.getenv("CLOUD_PROVIDER", conf.MOCK)
        os.environ["CLOUD_PROVIDER"] = conf.MOCK
        r = client.delete(
            conf.SERVICE_BASE_PATH + "/policies/osdu/partition/osdu/xyz.rego"
        )
        print(f"text: {r.text}")
        self.assertEqual(r.status_code, 200, "Not expected to be found")
        os.environ["CLOUD_PROVIDER"] = real_cloud_provider

    @set_authorize_session(ADMIN_ONLY_SVC, "osdu")
    def test_mock_put_all_policies_test_data(self):
        real_cloud_provider = os.getenv("CLOUD_PROVIDER", conf.MOCK)
        os.environ["CLOUD_PROVIDER"] = conf.MOCK
        testlib.put_policies_test_data(
            client=client, token="MOCK", data_partition="osdu"
        )
        os.environ["CLOUD_PROVIDER"] = real_cloud_provider


if __name__ == "__main__":
    logging.basicConfig(stream=sys.stderr)
    logging.getLogger("unittest").setLevel(logging.DEBUG)
    unittest.main()
